@@ -1,6 +1,6 @@
 package org.jmx4perl;
 
-import org.jmx4perl.handler.RequestHandler;
+import org.jmx4perl.handler.JsonRequestHandler;
 
 import javax.management.*;
 import javax.naming.InitialContext;
@@ -59,7 +59,7 @@ public class MBeanServerHandler {
      * @param pJmxReq the request to dispatch
      * @return the result of the request
      */
-    public Object dispatchRequest(RequestHandler pRequestHandler, JmxRequest pJmxReq)
+    public Object dispatchRequest(JsonRequestHandler pRequestHandler, JmxRequest pJmxReq)
             throws InstanceNotFoundException, AttributeNotFoundException, ReflectionException, MBeanException {
         if (pRequestHandler.handleAllServersAtOnce()) {
             return pRequestHandler.handleRequest(mBeanServers,pJmxReq);
@@ -250,7 +250,8 @@ public class MBeanServerHandler {
         // if ((isJBoss || isWebsphere)
         // The workaround was enabled for websphere as well, but it seems
         // to work without it for WAS 7.0
-        if (isJBoss && "java.lang".equals(pJmxReq.getObjectName().getDomain())) {
+        if (isJBoss && pJmxReq.getObjectName() != null &&
+                "java.lang".equals(pJmxReq.getObjectName().getDomain())) {
             try {
                 // invoking getMBeanInfo() works around a bug in getAttribute() that fails to
                 // refetch the domains from the platform (JDK) bean server (e.g. for MXMBeans)
