@@ -45,7 +45,8 @@ public class VersionHandler extends JsonRequestHandler {
     }
 
     @Override
-    public Object doHandleRequest(MBeanServer server, JmxRequest request) {
+    public Object doHandleRequest(MBeanServer server, JmxRequest request)
+            throws InstanceNotFoundException, AttributeNotFoundException, ReflectionException, MBeanException {
         return Version.getVersion();
     }
 }
