@@ -41,6 +41,14 @@ Write an attribute
 
 Execute an JMX operation 
 
+=item LIST
+
+List all MBeans available
+
+=item SEARCH
+
+Search for MBeans
+
 =item REGISTER_NOTIFICATION
 
 Register for a JMX notification (not supported yet)
@@ -69,6 +77,15 @@ So, to fetch the C<"used"> value only, specify C<used> as path within the
 request. You can access deeper nested values by building up a path with "/" as
 separator. This looks a bit like a simplified form of XPath.
 
+=item max_depth, max_objects, max_list_size
+
+With these objects you can restrict the size of the JSON structure
+returned. C<max_depth> gives the maximum nesting level of the JSON
+object,C<max_objects> returns the maximum number of objects to be returned in
+total and C<max_list_size> restrict the number of all arrays and collections
+(maps, lists) in the answer. Note, that you should use this restrictions if you
+are doing massive bulk operations.
+
 =back 
 
 =head1 METHODS
@@ -87,28 +104,27 @@ use Data::Dumper;
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = (
-           "READ","WRITE","EXEC","LIST",
-           "REGISTER_NOTIFICATION","REMOVE_NOTIFICATION"
+           "READ","WRITE","EXEC","LIST", "SEARCH",
+           "REGNOTIF","REMNOTIF"
           );
 
-
 use constant READ => "read";
 use constant WRITE => "write";
 use constant EXEC => "exec";
 use constant LIST => "list";
-use constant REGISTER_NOTIFICATION => "regnotif";
-use constant REMOVE_NOTIFICATION => "remnotif";
+use constant SEARCH => "search";
+use constant REGNOTIF => "regnotif";
+use constant REMNOTIF => "remnotif";
 
 my $TYPES = 
-{ map { $_ => 1 } (READ, WRITE, EXEC, LIST, 
-                   REGISTER_NOTIFICATION, REMOVE_NOTIFICATION) };
-
+{ map { $_ => 1 } (READ, WRITE, EXEC, LIST, SEARCH,
+                   REGNOTIF, REMNOTIF) };
 
 =item  $req = new JMX::Jmx4Perl::Request(....);
 
- $req = new JMX::Jmx4Perl::Request(READ,$mbean,$attribute,$path);
- $req = new JMX::Jmx4Perl::Request(READ,{ mbean => $mbean,... } );
- $req = new JMX::Jmx4Perl::Request({type => READ, mbean => $mbean, ... } );
+ $req = new JMX::Jmx4Perl::Request(READ,$mbean,$attribute,$path, { ... options ... } );
+ $req = new JMX::Jmx4Perl::Request(READ,{ mbean => $mbean,... });
+ $req = new JMX::Jmx4Perl::Request({type => READ, mbean => $mbean, ... });
 
 The constructor can be used in various way. In the simplest form, you provide
 the type as first argument and depending on the type one or more additional
@@ -117,6 +133,10 @@ parameter and a hashref containing named parameter for the request parameters
 (for the names, see above). Finally you can specify the arguments completely as
 a hashref, using 'type' for the entry specifying the request type.
 
+For the options C<max_depth>, C<max_objects> and C<max_list_size>, you can mix
+them in into the hashref if using the hashed argument format. For the first
+format, these options are given as a final hashref.
+
 Note, depending on the type, some parameters are mandatory. The mandatory
 parameters and the order of the arguments for the constructor variant without
 named parameters are:
@@ -143,6 +163,11 @@ named parameters are:
   
  Order    : $path
 
+=item C<SEARCH>
+
+ Order    : $pattern
+ Mandatory: $pattern
+
 =back
 
 =cut
@@ -167,6 +192,12 @@ sub new {
         } else {
             # Unnamed arguments
             $self = {type =>  $type};
+            my $opts = $_[scalar(@_)-1];
+            if (ref($opts) eq "HASH") {
+                pop @_;
+                map { $self->{$_} = $opts->{$_} } keys %$opts;
+            } else {
+            }
             if ($type eq READ) {
                 $self->{mbean} = shift;
                 $self->{attribute} = shift;
@@ -182,6 +213,8 @@ sub new {
                 $self->{args} = [ @_ ];
             } elsif ($type eq LIST) {
                 $self->{path} = shift;
+            } elsif ($type eq SEARCH) {
+                $self->{mbean} = shift;
             } else {
                 croak "Type ",$type," not supported yet";
             }
@@ -208,7 +241,7 @@ sub get {
 sub _validate {
     my $self = shift;
     if ($self->{type} eq READ ||  $self->{type} eq WRITE) {
-        die $self->{type} . ": No mbean name given\n" unless $self->{mbean};
+        die $self->{type} . ": No mbean name given\n",Dumper($self) unless $self->{mbean};
         die $self->{type} . ": No attribute name given\n" unless $self->{attribute};
     }
     if ($self->{type} eq WRITE) {
@@ -220,6 +253,20 @@ sub _validate {
     }
 }
 
+# Called for post requests
+sub TO_JSON {
+    my $self = shift;
+    return {
+            type => $self->{type} ? uc($self->{type}) : undef,
+            mbean => $self->{mbean},
+            attribute => $self->{attribute},
+            path => $self->{path},
+            arguments => $self->{args},
+            value => $self->{value},
+            operation => $self->{operation},
+           };
+}
+
 =back 
 
 =head1 LICENSE
